#ifndef CAPAFIGYELO_H
#define CAPAFIGYELO_H

/**
 * file: capafigyelo.h
 *
 * CapaFigyelo s FigyeltCapa osztlyok deklarcija
 *
 * A nagy inline fggvnyeket ILLENE kln definilni,
 * de gy taln jobban megrthet a megolds.
 *
 */

#include <list>
#include <stdexcept>
#include "halak.h"
#include "observer.h"

/// FigyeltCapa
class FigyeltCapa :public Capa, public Subject {
    Koord lastPos;
public:
    /// Pozci lekrdezse
    Koord getpos() const { return lastPos; }

    /// Viselkedst megvalst fggvny.
    /// Minden lps eltt jelez
    void lep(const Koord& pos, Ocean& oc, int it) {
        lastPos = pos;
        notify(1);      // jelez a megfigyelnek
        Capa::lep(pos, oc, it);
    }
};

/// CapaFigyel
/// Az egyszersg kedvrt csak egy tnyleges megfigyelnk lesz, ami
/// minden feladatot ellt, de szt lehetne szedni tbb osztlyra a feladatokat.
class CapaFigyelo : public Observer {
    int kor;
    int ehes;
    int szapor;
    std::list<Koord> utvonal;
public:
    CapaFigyelo(FigyeltCapa *fc) :Observer(fc), kor(0), ehes(0), szapor(0) {}

    /// Kor kiolvassa
    int getkor() const { return kor; }

    /// Kor kiolvassa
    int getehes() const { return ehes; }

    /// Hnyszor szaporodott
    int getszapor() const { return szapor; }

    /// Update megvalstsa az Observer szmra
    void update(Subject *subj, int oka) {
        // Ha az oka == 0, akkor mr ppen nincs FigyeltCapa, ezrt nem krdezhetnk le tle semmit.
        if (oka != 0) {
            // Tudjuk, hogy ez egy figyelt cpra mutat pointer, ezrt
            // tkonvertlhatjuk. A dynamic_cast radsul ellenrzi is.
            FigyeltCapa *fc = dynamic_cast<FigyeltCapa*>(subj);

            // Biztonsgbl ellenrizzk a cast eredmnyt
            if (fc == 0) throw std::runtime_error("bad_cast: FigyeltCapa");

            // sikerlt, kibnysszuk a szksges informcikat
            kor = fc->getkor();
            ehes = fc->getehes();
            szapor = fc->getszapor();
            utvonal.push_back(fc->getpos());
        }
        Observer::update(subj, oka);// Alaposztly update: megsznteti a regisztrcit, ha kell
    }

    /// Kiirja a letrolt utat
    void ut(std::ostream& os) {
        for (std::list<Koord>::iterator it = utvonal.begin(); it != utvonal.end(); it++)
            os << "(" << it->i << "," << it->j << ") ";
    }
};

#endif

