/**
 * file: main.cpp
 *
 * Program az cenmodell tesztelshez.
 *
 */
#include <iostream>
#include <stdexcept>
#include <algorithm>
#include <functional>
#include <vector>
#include "memtrace.h"
#include "ocean.h"
#include "halor.h"
#include "capafigyelo.h"

using std::cout;
using std::cerr;
using std::endl;

/// Halneveket sszehasonlt fv. objektum
struct HalnevCmp {
    char refnev;            // referencia nv

    /// Konstruktor letrolja a referencia karaktart
    /// @param nev - referencianv
    HalnevCmp(char nev) :refnev(nev) {}

    /// Referencia sszehasonltsa
    /// @param o - Obj objektum
    /// @return - true, ha az objektum neve megegyezik a referencianvvel
    bool operator()(const Obj* o) const {
        return o->getnev() == refnev;
    }
};

/// Halr statikus tag definci
int Halor::darab = 0;


int main() {
  try {
    Ocean atlanti;

    FigyeltCapa *capeti = new FigyeltCapa;
    atlanti.replObj(Koord(2,5), capeti);
    atlanti.replObj(Koord(0,0), new Hal);
    atlanti.replObj(Koord(5,10), new Hal);
    atlanti.replObj(Koord(MaxN-1,MaxM-1), new Hal);
    CapaFigyelo mester(capeti);
    CapaFigyelo filmes(capeti);
    CapaFigyelo biologus(capeti);

    for (int i = 0; i < 54; i++) {
       cout << endl << i+1 << ". iteracio elott. Allatok szama:" << Halor::getDarab();

       // Halak szma
       cout << " (Hal:" << (int)count_if(atlanti.begin(), atlanti.end(), HalnevCmp('H'));

       // Cpk szma
       cout << ", Capa:" << (int)count_if(atlanti.begin(), atlanti.end(),
                                                                   HalnevCmp('C')) << ")" << endl;
       cout << "Capeti ehes:" << biologus.getehes() << endl;
       atlanti.rajzol(cout);
       atlanti.lep();
    }
    cout << "Capeti kora:" << mester.getkor() << endl;
    cout << "Capeti utodai:" << biologus.getszapor() << endl;
    cout << "Capeti utja:";
    filmes.ut(cout);
    cout << endl;

  } catch (std::exception &e) {
      cerr << "Nem vart kivtel: " << e.what() << endl;
  }                         // minden objektum megsznik
}

