/**
 * file: observer.cpp
 *
 * Observer osztly definicii
 *
 * Fontos:
 *  Amikor a megfigyelt osztly megsznik, a Subject osztly
 *  destruktora notyfy(0) zenetet kld. Ez az egyetlen zenet, amit
 *  a megolds fenntart bels adminisztrcis clokra.
 *  Ekkor a Subject-bl szrmaztatott pldny adatai mr nem lteznek,
 *  teht nem krdezheti le azokat a megfigyel.
 *
 */
#include <set>
#include "observer.h"
#include "memtrace.h"

/// Observer bejelentkezik
/// @param os - rdekld observer
void Subject::attach(Observer *o) {
    obs.insert(o);
}

/// Observer kijelentkezik
/// @param os - observer, aki mr nem kvncsi tovbb
void Subject::detach(Observer *o) {
    obs.erase(obs.find(o));
}

/// Megfigyelend pldny llapota vltozott
/// @param reason - mi vltotta ki a vltozst (0- megsznt a pldny)
void Subject::notify(int reason) {
    for (std::set<Observer*>::iterator it = obs.begin(); it != obs.end(); it++ )
        (*it)->update(this, reason);    // mindenkinek zen
}

/// Megsznik a megfigyelend
Subject::~Subject() {
        notify(0);          // jelzi, hogy megsznt
    }

/// Konstruktor regisztrl (attach)
Observer::Observer(Subject *subj) :subj(subj){
    subj->attach(this);
}

/// Megfigyelt pldny vltozott
/// @param subj - megfigyelt pldny pointere
/// @param reason - mi vltotta ki a vltozs
///                 0: megsznt, a tbbi kd szabad
void Observer::update(Subject* subj, int reason) {
    if (reason == 0)
        this->subj = 0;     // megsznt! Nem tudjuk tovbb figyelni
}

/// Destruktor kiregisztrl (detach)
Observer::~Observer() {
    if (subj != 0)          // van mg kit figyelni ?
        subj->detach(this);
}

