#ifndef OBSERVER_H
#define OBSERVER_H

/**
 * file: observer.h
 *
 * Observer osztly deklarcii
 *
 * Egy-sok kapcsolat pl fel Subject s az Observer
 * osztly pldnyai kztt.
 *
 */

#include <set>

class Observer;                 // eldeklarci

/// Subject (megfigyelend) osztly se
class Subject {
    std::set<Observer*> obs;    // Megfigyelk halmaza
public:
    /// Observer bejelentkezik
    /// @param os - rdekld observer
    void attach(Observer* os);

    /// Observer kijelentkezik
    /// @param os - observer, aki mr nem kvncsi tovbb
    void detach(Observer* os);

    /// Megfigyelend pldny llapota vltozott
    /// @param reason - mi vltotta ki a vltozst
    ///                 0: megsznt, a tbbi kd szabad
    void notify(int reason);

    /// Megsznik a megfigyelend
    virtual ~Subject();         // Virtulis, hogy az alaposztly fell is mkdjn
};

/// Observer (megfigyel) osztly se
class Observer {
    Subject *subj;              // megfigyelt objektum cme
public:
    /// konstruktor regisztrl (attach)
    /// @param subj - megfigyelt pldny pointere
    Observer(Subject* subj);

    /// Megfigyelt pldny vltozott
    /// @param subj - megfigyelt pldny pointere
    /// @param reason - mi vltotta ki a vltozst
    ///                 0: megsznt, a tbbi kd szabad
    virtual void update(Subject* subj, int reason);

    /// destruktor kiregisztrl (detach)
    virtual ~Observer();
};

#endif

