#ifndef OCEAN_H
#define OCEAN_H

/**
 * file: ocean.h
 *
 * Ocean objektum deklarcija
 *
 */

#include <iostream>
#include "obj.h"

/// cen mrete.
const int MaxN = 10;
const int MaxM = 40;

class Obj;		// Eldeklarci. (A krkrs include miatt kell.)

/// Ocean objektum.
/// Statikus mret cellarcsot tartalmaz.
/// Minden cella egy objektum mutatjt trolja.
class Ocean {
    int iter;                   /// Iterci szmllja
    Obj *cellak[MaxN][MaxM];    /// Cellk trolja
public:
	class Iterator;             // eldeklarci

	/// Deafult konstruktor
    Ocean();

    /// Eldnti egy pozicirl, hogy az rvnyes ocen pozici-e
    /// @return - true, ha rvnyes
    bool ervenyes(const Koord& k) const {
        return k.i >= 0 && k.i < MaxN && k.j >= 0 && k.j < MaxM;
	}

    /// Cellban trolt pointer lekrdezse.
    /// @param pos - cellapozci
    /// @return objektum pointere
    Obj* getObj(Koord pos) const;

    /// Cellban trolt pointer bersa.
    /// @param pos - cellapozci
    /// @param o - j objektum pointere
    void setObj(Koord pos, Obj* o);

    /// Cellban trolt pointer kicserlse.
    /// A rgi objektum felszabadul
    /// @param pos - cellapozci
    /// @param o - j objektum pointere
    void replObj(Koord pos, Obj* o);

    /// Cellarcsok "kirajzolsa"
    /// @param os - output stream
    void rajzol(std::ostream& os) const;

    /// Egy itercis lps
    void lep();

    /// Kezdrtk
    Iterator begin() { return Iterator(*this); }

    /// Vgrtk
    Iterator end() { return Iterator(*this, MaxN*MaxM); }

    /// cen itertora
    /// Az std::iterator osztlybl szrmaztatjuk, hogy hasznlhat legyen az algoritmusokban.
    /// Az std::itarator valjban csak tpusokat definil, amit az algoritmusokat megvalst
    /// sablonok hasznlnak.
    class Iterator :public std::iterator<std::input_iterator_tag, Obj> {
        Obj **p;        // aktulis pointer
        Obj **pe;       // vgpointer
    public:
        /// Inicializlatlan itertor
        Iterator() :p(0), pe(0) {}

        /// Konstruktor a begin-hez s az end-hez
        /// @param o - ocen
        /// @param n - akt. index
        Iterator(Ocean& o, int n = 0)
            :p(&o.cellak[0][0]+n), pe(&o.cellak[0][0]+MaxN*MaxM) {}

        /// !=
        bool operator!=(Iterator& it) {
            return p != it.p;
        }

        /// ==
        bool operator==(Iterator& it) {
            return p == it.p;
        }

        /// Pre inkremens
        Iterator& operator++();

        /// Post inkremens
        Iterator operator++(int);

        /// Csillag
        Obj* operator*();

        /// Nyil opertor
        Obj** operator->();
    };

    ~Ocean();
};
#endif
