/**
 * \file gen_array_adapter.hpp
 *
 * Generikus tömb std::vector-ból
 *  Előadáson bemutatott kód módosított változata.
 */
#ifndef GEN_ARRAY_ADAPTER
#define GEN_ARRAY_ADAPTER

#include <vector>
#include <stdexcept>

/**
 * Fix méretű generikus tömb.
 * @param T - tárolt adattípus
 * @param maxsiz - maximális méret
 */
template <typename T, size_t maxsiz = 6>
class Array : public std::vector<T> {
    T& operator[](size_t i);
    const T& operator[](size_t i) const;

public:
    Array(size_t n = 0, const T& value = T())
        :std::vector<T>::vector(std::min(n,maxsiz), value) {
        std::vector<T>::reserve(maxsiz);
    }

    template <class Iter>
    Array(Iter first, Iter last)
        :std::vector<T>::vector(first, last) {
        if (std::vector<T>::size() > maxsiz)
            std::vector<T>::resize(maxsiz);
        else
            std::vector<T>::reserve(maxsiz);
    }

    T& at(size_t i) {
        if (i < maxsiz && i >= std::vector<T>::size())
            std::vector<T>::resize(i+1);
        return std::vector<T>::at(i);
    }

    const T& at(size_t i) const {
        return std::vector<T>::at(i);
    }

}; // generikus tömb vége

#endif
